use TFC

if(exists(select * from sys.triggers where name = 'trg_vendes'))
	drop trigger [dbo].[trg_vendes]
	go

create trigger [dbo].[trg_vendes]
	on tw_vendes
	after insert, update, delete 
as
begin
	set nocount on
	
	-- comprova tipus d'acci
	declare @accio as char(1)
	set @accio = (case when exists(select * from inserted) and exists(select * from deleted) then 'U' --actualitzaci
					  when exists(select * from inserted) then 'I' -- inserci
					  when exists(select * from deleted) then 'D' -- eliminaci
					  else null -- acci no vlida
				  end)
				  
	--definim les variables auxiliars
	declare @id_botiga int,
			@identificador_ean13 nvarchar(13),
			@id_poblacio int,
			@data date,
			@hora int,
			@quantitat_total int,
			@preu_brut_total decimal(18,2),
			@benefici_net_total decimal(18,2),
			@baixa datetime
			
	-- carreguem les dades a les variables auxiliars
	if(@accio = 'I' or @accio = 'U')
	begin
		select @id_botiga = id_botiga, @identificador_ean13 = identificador_ean13, @data = data_venda,
			   @hora = hora_venda, @quantitat_total = quantitat_total, @preu_brut_total = preu_brut_total, 
			   @benefici_net_total = benefici_net_total, @baixa = baixa
		from INSERTED
	end
	else
	begin			
		select @id_botiga = id_botiga, @identificador_ean13 = identificador_ean13, @data = data_venda,
			   @hora = hora_venda, @quantitat_total = quantitat_total, @preu_brut_total = preu_brut_total, 
			   @benefici_net_total = benefici_net_total, @baixa = baixa
		from DELETED
	end
	
	--cerquem la poblacio arran de la botiga
	set @id_poblacio = (select id_poblacio from tw_botiga where id_botiga = @id_botiga)
	
	declare @id_st int -- auxiliar per agafar l'identificador de la taula
	declare @any int = year(@data) --agafem l'any de la venda
	
	----------------------------------------------------------------------------------------------------------------------
	-- st_benefici_net
	----------------------------------------------------------------------------------------------------------------------
	-- reiniciem el historic
	update st_benefici_net set any_mes_benefici = 0	
	
	-- cerquem si n'hi ha una entrada pel any especificat
	select @id_st = id_benefici_net from st_benefici_net where [any] = @any
	if(@id_st is null)
	begin
		-- inserim un nou any
		insert into st_benefici_net ([any], benefici_net, any_mes_benefici)
		select @any, isnull(sum(benefici_net_total), 0), 0
		from tw_vendes 
		where YEAR(data_venda) = @any 
			and baixa is null
	end
	else
	begin
		update st_benefici_net set benefici_net = isnull((select SUM(benefici_net_total) from tw_vendes where YEAR(data_venda) = @any), 0) where [any] = @any
	end
	
	-- cerquem quin es l'any amb ms vendes
	declare @any_historic int = (select top 1 [any] from st_benefici_net t inner join (select max(benefici_net) benefici_net from st_benefici_net) tt on tt.benefici_net = t.benefici_net)
	update st_benefici_net set any_mes_benefici = 1 where [any] = @any_historic
	
	----------------------------------------------------------------------------------------------------------------------
	-- st_botiga_mes_benefici
	----------------------------------------------------------------------------------------------------------------------
	-- reiniciem el historic
	update st_botiga_mes_benefici set any_mes_benefici = 0
	
	-- cerquem si n'hi ha una entrada pel any especificat
	select @id_st = id_botiga_benefici from st_botiga_mes_benefici where [any] = @any
	if(@id_st is null)
	begin
		-- inserim un nou any
		insert into st_botiga_mes_benefici ([any], id_botiga, benefici_net, any_mes_benefici)
		select @any, @id_botiga, isnull(sum(benefici_net_total), 0), 0
		from tw_vendes 
		where YEAR(data_venda) = @any 
			and id_botiga = @id_botiga
			and baixa is null
	end
	else
	begin
		-- obtenim el benefici actual i el calculat per la botiga
		declare @aux_ben decimal(18,2), @aux_bot int
		select @aux_ben = benefici_net, @aux_bot = id_botiga from st_botiga_mes_benefici where [any] = @any
		declare @aux2_ben decimal(18,2)
		select @aux2_ben = sum(benefici_net_total) from tw_vendes where year(data_venda) = @any and id_botiga = @id_botiga and baixa is null
		
		-- si el total del benefici de la botiga es ms alt que el actual, sustituim
		if @aux2_ben > @aux_ben
			update st_botiga_mes_benefici set id_botiga = @id_botiga, benefici_net = @aux2_ben where id_botiga_benefici = @id_st
		else
		begin
			-- si es la mateixa botiga i el benefici es menor, calculem quin botiga t el benefici ms alt i sustituim
			if @aux_bot = @id_botiga and @aux2_ben < @aux_ben
			begin
				update st_botiga_mes_benefici set id_botiga = v.id_botiga, benefici_net = v.benefici_net
				from st_botiga_mes_benefici st, (
					select top 1 id_botiga, sum(benefici_net_total) benefici_net
					from tw_vendes 
					where year(data_venda) = @any
						and baixa is null
					group by id_botiga
					order by benefici_net desc
				) v
				where st.id_botiga_benefici = @id_st
			end
		end
	end
	
	-- cerquem quin es l'any amb ms vendes
	set @any_historic = (select top 1 [any] from st_botiga_mes_benefici t inner join (select max(benefici_net) benefici_net from st_botiga_mes_benefici) tt on tt.benefici_net = t.benefici_net)
	update st_botiga_mes_benefici set any_mes_benefici = 1 where [any] = @any_historic
	
	
	----------------------------------------------------------------------------------------------------------------------
	-- st_producte_mes_venut
	----------------------------------------------------------------------------------------------------------------------
	-- reiniciem el historic
	update st_producte_mes_venut set any_producte_mes_venut = 0
	
	-- cerquem si n'hi ha una entrada pel any especificat
	select @id_st = id_producte_venut from st_producte_mes_venut where [any] = @any
	if(@id_st is null)
	begin
		-- inserim un nou any
		insert into st_producte_mes_venut ([any], identificador_ean13, quantitat, any_producte_mes_venut)
		select @any, @identificador_ean13, isnull(sum(quantitat_total), 0), 0
		from tw_vendes 
		where YEAR(data_venda) = @any 
			and identificador_ean13 = @identificador_ean13
			and baixa is null
	end
	else
	begin
		declare @aux_quan decimal(18,2), @aux_prod nvarchar(13)
		-- obtenim la quantitat de vendes actual i el calculat per el producte
		select @aux_quan = quantitat, @aux_prod = identificador_ean13 from st_producte_mes_venut where [any] = @any
		declare @aux2_quan decimal(18,2)
		select @aux2_quan = sum(quantitat_total) from tw_vendes where year(data_venda) = @any and identificador_ean13 = @identificador_ean13 and baixa is null

		-- si el total de la quantitat de vendes del producte es ms alt que el actual, sustituim
		if @aux2_quan > @aux_quan
			update st_producte_mes_venut set identificador_ean13 = @identificador_ean13, quantitat = @aux2_ben where id_producte_venut = @id_st
		else
		begin
			-- si es el mateix producte i la quantitat es menor, calculem quin producte t ms vendes i sustituim
			if @aux_prod = @identificador_ean13 and @aux2_quan < @aux_quan
			begin
				update st_producte_mes_venut set identificador_ean13 = v.identificador_ean13, quantitat = v.quantitat
				from st_producte_mes_venut st, (
					select top 1 identificador_ean13, sum(quantitat_total) quantitat
					from tw_vendes 
					where year(data_venda) = @any
						and baixa is null
					group by identificador_ean13
					order by quantitat desc
				) v
				where st.id_producte_venut = @id_st
			end
		end
	end
	
	-- cerquem quin es l'any amb ms vendes
	set @any_historic = (select top 1 [any] from st_producte_mes_venut t inner join (select max(quantitat) quantitat from st_producte_mes_venut) tt on tt.quantitat = t.quantitat)
	update st_producte_mes_venut set any_producte_mes_venut = 1 where [any] = @any_historic
	
	----------------------------------------------------------------------------------------------------------------------
	-- st_hora_mes_vendes
	----------------------------------------------------------------------------------------------------------------------
	-- reiniciem el historic
	update st_hora_mes_vendes set any_hora_mes_vendes = 0
	
	-- cerquem si n'hi ha una entrada pel any especificat
	select @id_st = id_hora_mes_vendes from st_hora_mes_vendes where [any] = @any
	if(@id_st is null)
	begin
		-- inserim un nou any
		insert into st_hora_mes_vendes ([any], hora, xifra_vendes, any_hora_mes_vendes)
		select @any, @hora, isnull(sum(quantitat_total), 0), 0
		from tw_vendes 
		where YEAR(data_venda) = @any 
			and hora_venda = @hora
			and baixa is null
	end
	else
	begin
		declare @aux_vendes_mes int, @aux_hora_mes int
		-- obtenim la quantitat de vendes actual i el calculat per el producte
		select @aux_vendes_mes = xifra_vendes, @aux_hora_mes = hora from st_hora_mes_vendes where [any] = @any
		declare @aux2_vendes_mes int
		select @aux2_vendes_mes = sum(quantitat_total) from tw_vendes where year(data_venda) = @any and hora_venda = @hora and baixa is null

		-- si el total de la quantitat de vendes en una hora es ms alt que el actual, sustituim
		if @aux2_vendes_mes > @aux_vendes_mes
			update st_hora_mes_vendes set hora = @hora, xifra_vendes = @aux2_vendes_mes where id_hora_mes_vendes = @id_st
		else
		begin
			-- si es la mateixa hora i la quantitat es menor, calculem quina hora t ms vendes i sustituim
			if @aux_hora_mes = @hora and @aux2_vendes_mes < @aux_vendes_mes
			begin
				update st_hora_mes_vendes set hora = v.hora_venda, xifra_vendes = v.quantitat
				from st_hora_mes_vendes st, (
					select top 1 hora_venda, sum(quantitat_total) quantitat
					from tw_vendes 
					where year(data_venda) = @any
						and baixa is null
					group by hora_venda
					order by quantitat desc
				) v
				where st.id_hora_mes_vendes = @id_st
			end
		end
	end
	
	-- cerquem quin es l'any amb ms vendes
	set @any_historic = (select top 1 [any] from st_hora_mes_vendes t inner join (select max(xifra_vendes) xifra_vendes from st_hora_mes_vendes) tt on tt.xifra_vendes = t.xifra_vendes)
	update st_hora_mes_vendes set any_hora_mes_vendes = 1 where [any] = @any_historic
	
	----------------------------------------------------------------------------------------------------------------------
	-- st_hora_menys_vendes
	----------------------------------------------------------------------------------------------------------------------
	-- reiniciem el historic
	update st_hora_menys_vendes set any_hora_menys_vendes = 0
	
	-- cerquem si n'hi ha una entrada pel any especificat
	select @id_st = id_hora_menys_vendes from st_hora_menys_vendes where [any] = @any
	if(@id_st is null)
	begin
		-- inserim un nou any
		insert into st_hora_menys_vendes ([any], hora, xifra_vendes, any_hora_menys_vendes)
		select @any, @hora, isnull(sum(quantitat_total), 0), 0
		from tw_vendes 
		where YEAR(data_venda) = @any 
			and hora_venda = @hora
			and baixa is null
	end
	else
	begin
		declare @aux_vendes_menys int, @aux_hora_menys int
		-- obtenim la quantitat de vendes actual i el calculat per el producte
		select @aux_vendes_menys = xifra_vendes, @aux_hora_menys = hora from st_hora_menys_vendes where [any] = @any
		declare @aux2_vendes_menys int
		select @aux2_vendes_menys = sum(quantitat_total) from tw_vendes where year(data_venda) = @any and hora_venda = @hora and baixa is null

		-- si el total de la quantitat de vendes en una hora es ms baix que el actual, sustituim
		if @aux2_vendes_menys < @aux_vendes_menys
			update st_hora_menys_vendes set hora = @hora, xifra_vendes = @aux2_vendes_menys where id_hora_menys_vendes = @id_st
		else
		begin
			-- si es la mateixa hora i la quantitat es ms gran, calculem quina hora t menys vendes i sustituim
			if @aux_hora_menys = @hora and @aux2_vendes_menys > @aux_vendes_menys
			begin
				update st_hora_menys_vendes set hora = v.hora_venda, xifra_vendes = v.quantitat
				from st_hora_menys_vendes st, (
					select top 1 hora_venda, sum(quantitat_total) quantitat
					from tw_vendes 
					where year(data_venda) = @any
						and baixa is null
					group by hora_venda
					order by quantitat asc
				) v
				where st.id_hora_menys_vendes = @id_st
			end
		end
	end
	
	-- cerquem quin es l'any amb menys vendes
	set @any_historic = (select top 1 [any] from st_hora_menys_vendes t inner join (select min(xifra_vendes) xifra_vendes from st_hora_menys_vendes) tt on tt.xifra_vendes = t.xifra_vendes)
	update st_hora_menys_vendes set any_hora_menys_vendes = 1 where [any] = @any_historic
	
	----------------------------------------------------------------------------------------------------------------------
	-- st_dia_mes_vendes
	----------------------------------------------------------------------------------------------------------------------
	-- reiniciem el historic
	update st_dia_mes_vendes set any_dia_mes_vendes = 0
	
	-- cerquem si n'hi ha una entrada pel any especificat
	select @id_st = id_dia_mes_vendes from st_dia_mes_vendes where [any] = @any
	if(@id_st is null)
	begin
		-- inserim un nou any
		insert into st_dia_mes_vendes ([any], dia, xifra_vendes, any_dia_mes_vendes)
		select @any, DAY(@data), isnull(sum(quantitat_total), 0), 0
		from tw_vendes 
		where YEAR(data_venda) = @any 
			and DAY(data_venda) = DAY(@data)
			and baixa is null
	end
	else
	begin
		declare @aux_vendesdia_mes int, @aux_dia_mes int
		-- obtenim la quantitat de vendes actual i el calculat per el dia
		select @aux_vendesdia_mes = xifra_vendes, @aux_dia_mes = dia from st_dia_mes_vendes where [any] = @any
		declare @aux2_vendesdia_mes int
		select @aux2_vendesdia_mes = sum(quantitat_total) from tw_vendes where year(data_venda) = @any and DAY(data_venda) = DAY(@data) and baixa is null

		-- si el total de la quantitat de vendes en un dia es ms alt que el actual, sustituim
		if @aux2_vendesdia_mes > @aux_vendesdia_mes
			update st_dia_mes_vendes set dia = DAY(@data), xifra_vendes = @aux2_vendesdia_mes where id_dia_mes_vendes = @id_st
		else
		begin
			-- si es el mateix dia i la quantitat es menor, calculem quin dia t ms vendes i sustituim
			if @aux_dia_mes = DAY(@data) and @aux2_vendesdia_mes < @aux_vendesdia_mes
			begin
				update st_dia_mes_vendes set dia = v.data_venda, xifra_vendes = v.quantitat
				from st_dia_mes_vendes st, (
					select top 1 DAY(data_venda) data_venda, sum(quantitat_total) quantitat
					from tw_vendes 
					where year(data_venda) = @any
						and baixa is null
					group by DAY(data_venda)
					order by quantitat desc
				) v
				where st.id_dia_mes_vendes = @id_st
			end
		end
	end
	
	-- cerquem quin es l'any amb ms vendes
	set @any_historic = (select top 1 [any] from st_dia_mes_vendes t inner join (select max(xifra_vendes) xifra_vendes from st_dia_mes_vendes) tt on tt.xifra_vendes = t.xifra_vendes)
	update st_dia_mes_vendes set any_dia_mes_vendes = 1 where [any] = @any_historic
	
	----------------------------------------------------------------------------------------------------------------------
	-- st_dia_menys_vendes
	----------------------------------------------------------------------------------------------------------------------
	-- reiniciem el historic
	update st_dia_menys_vendes set any_dia_menys_vendes = 0
	
	-- cerquem si n'hi ha una entrada pel any especificat
	select @id_st = id_dia_menys_vendes from st_dia_menys_vendes where [any] = @any
	if(@id_st is null)
	begin
		-- inserim un nou any
		insert into st_dia_menys_vendes ([any], dia, xifra_vendes, any_dia_menys_vendes)
		select @any, DAY(@data), isnull(sum(quantitat_total), 0), 0
		from tw_vendes 
		where YEAR(data_venda) = @any 
			and DAY(data_venda) = DAY(@data)
			and baixa is null
	end
	else
	begin
		declare @aux_vendesdia_menys int, @aux_dia_menys int
		-- obtenim la quantitat de vendes actual i el calculat per el dia
		select @aux_vendesdia_menys = xifra_vendes, @aux_dia_menys = dia from st_dia_menys_vendes where [any] = @any
		declare @aux2_vendesdia_menys int
		select @aux2_vendesdia_menys = sum(quantitat_total) from tw_vendes where year(data_venda) = @any and DAY(data_venda) = DAY(@data) and baixa is null

		-- si el total de la quantitat de vendes en un dia es ms baix que el actual, sustituim
		if @aux2_vendesdia_mes > @aux_vendesdia_mes
			update st_dia_menys_vendes set dia = DAY(@data), xifra_vendes = @aux2_vendesdia_menys where id_dia_menys_vendes = @id_st
		else
		begin
			-- si es el mateix dia i la quantitat es ms gran, calculem quin dia t menys vendes i sustituim
			if @aux_dia_menys = DAY(@data) and @aux2_vendesdia_menys > @aux_vendesdia_menys
			begin
				update st_dia_menys_vendes set dia = v.data_venda, xifra_vendes = v.quantitat
				from st_dia_menys_vendes st, (
					select top 1 DAY(data_venda) data_venda, sum(quantitat_total) quantitat
					from tw_vendes 
					where year(data_venda) = @any
						and baixa is null
					group by DAY(data_venda)
					order by quantitat asc
				) v
				where st.id_dia_menys_vendes = @id_st
			end
		end
	end
	
	-- cerquem quin es l'any amb menys vendes
	set @any_historic = (select top 1 [any] from st_dia_menys_vendes t inner join (select min(xifra_vendes) xifra_vendes from st_dia_menys_vendes) tt on tt.xifra_vendes = t.xifra_vendes)
	update st_dia_menys_vendes set any_dia_menys_vendes = 1 where [any] = @any_historic
	
	----------------------------------------------------------------------------------------------------------------------
	-- st_ciutat_mes_beneficis
	----------------------------------------------------------------------------------------------------------------------
	-- reiniciem el historic
	update st_ciutat_mes_beneficis set any_ciutat_mes_beneficis = 0
	
	-- cerquem si n'hi ha una entrada pel any especificat
	select @id_st = id_ciutat_mes_beneficis from st_ciutat_mes_beneficis where [any] = @any
	if(@id_st is null)
	begin
		-- inserim un nou any
		insert into st_ciutat_mes_beneficis ([any], id_poblacio, beneficis, any_ciutat_mes_beneficis)
		select @any, @id_poblacio, isnull(sum(benefici_net_total), 0), 0
		from tw_vendes v
			inner join tw_botiga b on b.id_botiga = v.id_botiga
		where YEAR(data_venda) = @any 
			and id_poblacio = @id_poblacio
			and v.baixa is null
	end
	else
	begin
		declare @aux_benepobla_mes decimal(18,2), @aux_poblacio_mes int
		-- obtenim el benefici actual i el calculat per botiga
		select @aux_benepobla_mes = beneficis, @aux_poblacio_mes = id_poblacio from st_ciutat_mes_beneficis where [any] = @any
		declare @aux2_benepobla_mes decimal(18,2)
		select @aux2_benepobla_mes = sum(benefici_net_total) from tw_vendes v inner join tw_botiga b on b.id_botiga = v.id_botiga where year(data_venda) = @any and id_poblacio = @id_poblacio and v.baixa is null

		-- si el benefici es ms alt que el actual, sustituim
		if @aux2_benepobla_mes > @aux_benepobla_mes
			update st_ciutat_mes_beneficis set id_poblacio = @id_poblacio, beneficis = @aux2_benepobla_mes where id_ciutat_mes_beneficis = @id_st
		else
		begin
			-- si es el mateixa poblacio i el benefici es menor, calculem quina poblacio t ms benfici i sustituim
			if @aux_poblacio_mes = @id_poblacio and @aux2_benepobla_mes < @aux_benepobla_mes
			begin
				update st_ciutat_mes_beneficis set id_poblacio = v.id_poblacio, beneficis = v.benefici_net_total
				from st_ciutat_mes_beneficis st, (
					select top 1 id_poblacio, sum(benefici_net_total) benefici_net_total
					from tw_vendes v
						inner join tw_botiga b on b.id_botiga = v.id_botiga
					where year(data_venda) = @any
						and v.baixa is null
					group by id_poblacio
					order by benefici_net_total desc
				) v
				where st.id_ciutat_mes_beneficis = @id_st
			end
		end
	end
	
	-- cerquem quin es l'any amb ms benefici
	set @any_historic = (select top 1 [any] from st_ciutat_mes_beneficis t inner join (select max(beneficis) beneficis from st_ciutat_mes_beneficis) tt on tt.beneficis = t.beneficis)
	update st_ciutat_mes_beneficis set any_ciutat_mes_beneficis = 1 where [any] = @any_historic
	
	----------------------------------------------------------------------------------------------------------------------
	-- st_ciutat_menys_beneficis
	----------------------------------------------------------------------------------------------------------------------
	-- reiniciem el historic
	update st_ciutat_menys_beneficis set any_ciutat_menys_beneficis = 0
	
	-- cerquem si n'hi ha una entrada pel any especificat
	select @id_st = id_ciutat_menys_beneficis from st_ciutat_menys_beneficis where [any] = @any
	if(@id_st is null)
	begin
		-- inserim un nou any
		insert into st_ciutat_menys_beneficis ([any], id_poblacio, beneficis, any_ciutat_menys_beneficis)
		select @any, @id_poblacio, isnull(sum(benefici_net_total), 0), 0
		from tw_vendes v
			inner join tw_botiga b on b.id_botiga = v.id_botiga
		where YEAR(data_venda) = @any 
			and id_poblacio = @id_poblacio
			and v.baixa is null
	end
	else
	begin
		declare @aux_benepobla_menys decimal(18,2), @aux_poblacio_menys int
		-- obtenim el benefici actual i el calculat per botiga
		select @aux_benepobla_menys = beneficis, @aux_poblacio_menys = id_poblacio from st_ciutat_menys_beneficis where [any] = @any
		declare @aux2_benepobla_menys decimal(18,2)
		select @aux2_benepobla_menys = sum(benefici_net_total) from tw_vendes v inner join tw_botiga b on b.id_botiga = v.id_botiga where year(data_venda) = @any and id_poblacio = @id_poblacio and v.baixa is null

		-- si el benefici es ms baix que el actual, sustituim
		if @aux2_benepobla_menys < @aux_benepobla_menys
			update st_ciutat_menys_beneficis set id_poblacio = @id_poblacio, beneficis = @aux2_benepobla_menys where id_ciutat_menys_beneficis = @id_st
		else
		begin
			-- si es el mateixa poblacio i el benefici s ms gran, calculem quina poblacio t menys benfici i sustituim
			if @aux_poblacio_menys = @id_poblacio and @aux2_benepobla_menys > @aux_benepobla_menys
			begin
				update st_ciutat_menys_beneficis set id_poblacio = v.id_poblacio, beneficis = v.benefici_net_total
				from st_ciutat_menys_beneficis st, (
					select top 1 id_poblacio, sum(benefici_net_total) benefici_net_total
					from tw_vendes v
						inner join tw_botiga b on b.id_botiga = v.id_botiga
					where year(data_venda) = @any
						and v.baixa is null
					group by id_poblacio
					order by benefici_net_total asc
				) v
				where st.id_ciutat_menys_beneficis = @id_st
			end
		end
	end
	
	-- cerquem quin es l'any amb menys benefici
	set @any_historic = (select top 1 [any] from st_ciutat_menys_beneficis t inner join (select min(beneficis) beneficis from st_ciutat_menys_beneficis) tt on tt.beneficis = t.beneficis)
	update st_ciutat_menys_beneficis set any_ciutat_menys_beneficis = 1 where [any] = @any_historic
	
end
go