use TFC

if(exists(select * from sys.objects where object_id = object_id(N'[dbo].[pa_botigues]')))
	drop procedure pa_botigues
if(exists(select * from sys.objects where object_id = object_id(N'[dbo].[pm_botigues]')))
	drop procedure pm_botigues
if(exists(select * from sys.objects where object_id = object_id(N'[dbo].[pb_botigues]')))
	drop procedure pb_botigues
if(exists(select * from sys.objects where object_id = object_id(N'[dbo].[pa_productes]')))
	drop procedure pa_productes
if(exists(select * from sys.objects where object_id = object_id(N'[dbo].[pm_productes]')))
	drop procedure pm_productes
if(exists(select * from sys.objects where object_id = object_id(N'[dbo].[pb_productes]')))
	drop procedure pb_productes
if(exists(select * from sys.objects where object_id = object_id(N'[dbo].[pa_vendes]')))
	drop procedure pa_vendes
if(exists(select * from sys.objects where object_id = object_id(N'[dbo].[pm_vendes]')))
	drop procedure pm_vendes
if(exists(select * from sys.objects where object_id = object_id(N'[dbo].[pb_vendes]')))
	drop procedure pb_vendes
go

create procedure [dbo].[pa_botigues]
	@id_poblacio int, -- identificador de la poblaci
	@email_gerent nvarchar(100), -- email del gerent de la botiga
	@nombre_treballadors int, -- nombre de treballador existents, sempre major o igual a 1
	@es_franquicia bit, -- indica si es franquicia o pertany a la cadena
	@es_virtual bit, -- indica si la botiga es virtual o fsica
	@rsp nvarchar(50) output -- cont el resultat de l'execuci
as
begin
	/*
	Descripci:
	El procediment s'encarrega de insertar una nova botiga al sistema, previament vlida 
	que s'hagi emplenat totes les dades i que aquestes siguin correctes
	*/
	
	set nocount on
	
	declare @rsp_trace nvarchar(50) -- cont la resposta del procediment trace
	
	-- obtenim la cadena amb els parametres d'entrada
	declare @params nvarchar(max) = 'id_poblacio: ' + cast(@id_poblacio as nvarchar) +
									', email_gerent: ' + isnull(@email_gerent, '') +
									', nombre_treballadors: ' + cast(@nombre_treballadors as nvarchar) +
									', es_franquicia: ' + cast(@es_franquicia as nvarchar) +
									', es_virtual: ' + cast(@es_virtual as nvarchar)
	
	begin try
		begin transaction
		
		-- validacions
		if(not exists(select * from tm_poblacio where id_poblacio = @id_poblacio))
			raiserror('Poblaci no existeix', 16, 1)
		
		if(@email_gerent is null or LEN(@email_gerent) = 0)
			raiserror('Es necessari l''email del gerent', 16, 1)
			
		if(@nombre_treballadors is null or @nombre_treballadors <= 0)			
			raiserror('El nombre de treballador no es vlid', 16, 1)
			
		-- inserci
		insert into tw_botiga (id_poblacio, email_gerent, nombre_treballadors, es_franquicia, es_virtual)
		values (@id_poblacio, @email_gerent, @nombre_treballadors, @es_franquicia, @es_virtual)
		
		if @@TRANCOUNT > 0
			commit transaction
			
		set @rsp = 'OK'
		
		exec trace 'pa_botigues', @params, 1, @rsp, @rsp_trace output
	end try
	begin catch
	
		if @@TRANCOUNT > 0
			rollback transaction		
		
		declare @error_message nvarchar(max) = error_message()
		set @rsp = 'ERROR ' + error_message()
		
		exec trace 'pa_botigues', @params, 0, @error_message, @rsp_trace output
		
	end catch
end
go

create procedure [dbo].[pm_botigues]
	@id_botiga int, -- identificador de la botiga a editar
	@id_poblacio int, -- identificador de la poblaci
	@email_gerent nvarchar(100), -- email del gerent de la botiga
	@nombre_treballadors int, -- nombre de treballador existents, sempre major o igual a 1
	@es_franquicia bit, -- indica si es franquicia o pertany a la cadena
	@es_virtual bit, -- indica si la botiga es virtual o fsica
	@rsp nvarchar(50) output -- cont el resultat de l'execuci
as
begin
	/*
	Descripci:
	El procediment s'encarrega de modificar una botiga existent del sistema, previament vlida 
	que s'hagi emplenat totes les dades i que aquestes siguin correctes
	*/
	
	set nocount on
	
	declare @rsp_trace nvarchar(50) -- cont la resposta del procediment trace
	
	-- obtenim la cadena amb els parametres d'entrada
	declare @params nvarchar(max) = 'id_botiga: ' + cast(@id_botiga as nvarchar) +
									',id_poblacio: ' + cast(@id_poblacio as nvarchar) +
									', email_gerent: ' + isnull(@email_gerent, '') +
									', nombre_treballadors: ' + cast(@nombre_treballadors as nvarchar) +
									', es_franquicia: ' + cast(@es_franquicia as nvarchar) +
									', es_virtual: ' + cast(@es_virtual as nvarchar)
	
	begin try
		begin transaction
		
		-- validacions
		if(not exists(select * from tw_botiga where id_botiga = @id_botiga))
			raiserror('Botiga no existeix', 16, 1)			
		
		if(not exists(select * from tw_botiga where id_botiga = @id_botiga and baixa is null))
			raiserror('Botiga ja esta donada de baixa', 16, 1)
		
		if(not exists(select * from tm_poblacio where id_poblacio = @id_poblacio))
			raiserror('Poblaci no existeix', 16, 1)
		
		if(@email_gerent is null or LEN(@email_gerent) = 0)
			raiserror('Es necessari l''email del gerent', 16, 1)
			
		if(@nombre_treballadors is null or @nombre_treballadors <= 0)			
			raiserror('El nombre de treballador no es vlid', 16, 1)
			
		-- edici
		update tw_botiga 
		set id_poblacio = @id_poblacio,
			email_gerent = @email_gerent,
			nombre_treballadors = @nombre_treballadors,
			es_franquicia = @es_franquicia,
			es_virtual = @es_virtual
		where id_botiga = @id_botiga
		
		if @@TRANCOUNT > 0
			commit transaction
			
		set @rsp = 'OK'
		
		exec trace 'pm_botigues', @params, 1, @rsp, @rsp_trace output
	end try
	begin catch
	
		if @@TRANCOUNT > 0
			rollback transaction		
		
		declare @error_message nvarchar(max) = error_message()
		set @rsp = 'ERROR ' + error_message()
		
		exec trace 'pm_botigues', @params, 0, @error_message, @rsp_trace output
		
	end catch
end
go

create procedure [dbo].[pb_botigues]
	@id_botiga int, -- identificador de la botiga a donar de baixa
	@rsp nvarchar(50) output -- cont el resultat de l'execuci
as
begin
	/*
	Descripci:
	El procediment s'encarrega de donar de baixa una botiga existent del sistema, previament vlida 
	que la botiga existeixi i estigui d'alta
	*/
	
	set nocount on
	
	declare @rsp_trace nvarchar(50) -- cont la resposta del procediment trace
	
	-- obtenim la cadena amb els parametres d'entrada
	declare @params nvarchar(max) = 'id_botiga: ' + cast(@id_botiga as nvarchar)
	
	begin try
		begin transaction
		
		-- validacions
		if(not exists(select * from tw_botiga where id_botiga = @id_botiga))
			raiserror('Botiga no existeix', 16, 1)
		
		if(not exists(select * from tw_botiga where id_botiga = @id_botiga and baixa is null))
			raiserror('Botiga ja esta donada de baixa', 16, 1)
			
		-- baixa
		update tw_botiga 
		set baixa = getdate()
		where id_botiga = @id_botiga
		
		if @@TRANCOUNT > 0
			commit transaction
			
		set @rsp = 'OK'
		
		exec trace 'pb_botigues', @params, 1, @rsp, @rsp_trace output
	end try
	begin catch
	
		if @@TRANCOUNT > 0
			rollback transaction
		
		declare @error_message nvarchar(max) = error_message()
		set @rsp = 'ERROR ' + error_message()
		
		exec trace 'pb_botigues', @params, 0, @error_message, @rsp_trace output
		
	end catch
end
go

create procedure [dbo].[pa_productes]
	@identificador_ean13 nvarchar(13), -- identificador del producte
	@descripcio nvarchar(200), -- descripci del producte
	@preu_brut_actual decimal(18,2), -- preu de venda del producte
	@benefici_net_actual decimal(18,2), -- benefici que aporta el producte
	@rsp nvarchar(50) output -- cont el resultat de l'execuci
as
begin
	/*
	Descripci:
	El procediment s'encarrega de insertar un nou producte al sistema, previament vlida 
	que s'hagi emplenat totes les dades i que aquestes siguin correctes
	*/
	
	set nocount on
	
	declare @rsp_trace nvarchar(50) -- cont la resposta del procediment trace
	
	-- obtenim la cadena amb els parametres d'entrada
	declare @params nvarchar(max) = 'identificador_ean13: ' + cast(isnull(@identificador_ean13, '') as nvarchar) +
									'descripcio: ' + @descripcio +
									'preu_brut_actual: ' + cast(@preu_brut_actual as nvarchar) +
									'benefici_net_actual: ' + cast(@benefici_net_actual as nvarchar)
	
	begin try
		begin transaction
		
		-- validacions
		if(@identificador_ean13 is null)
			raiserror('Es requereix el identificador EAN13 del producte', 16, 1)
			
		if(exists(select * from tw_producte where identificador_ean13 = @identificador_ean13))
			raiserror('El producte ja existeix', 16, 1)
			
		if(@descripcio is null or LEN(@descripcio) = 0)
			raiserror('Es requereix una descripci', 16, 1)
		
		if(@preu_brut_actual is null or @preu_brut_actual <= 0)
			raiserror('Es necessari indicar un preu de venda vlid', 16, 1)
			
		if(@benefici_net_actual is null or @benefici_net_actual <= 0)			
			raiserror('El producte a de tindr un benefici', 16, 1)
			
		-- inserci
		insert into tw_producte (identificador_ean13, descripcio, preu_brut_actual, benefici_net_actual, alta)
		values (@identificador_ean13, @descripcio, @preu_brut_actual, @benefici_net_actual, GETDATE())
		
		if @@TRANCOUNT > 0
			commit transaction
			
		set @rsp = 'OK'
		
		exec trace 'pa_botigues', @params, 1, @rsp, @rsp_trace output
	end try
	begin catch
	
		if @@TRANCOUNT > 0
			rollback transaction		
		
		declare @error_message nvarchar(max) = error_message()
		set @rsp = 'ERROR ' + error_message()
		
		exec trace 'pa_botigues', @params, 0, @error_message, @rsp_trace output
		
	end catch
end
go

create procedure [dbo].[pm_productes]
	@identificador_ean13 nvarchar(13), -- identificador del producte a modificar
	@descripcio nvarchar(200), -- descripci del producte
	@preu_brut_actual decimal(18,2), -- preu de venda del producte
	@benefici_net_actual decimal(18,2), -- benefici que aporta el producte
	@rsp nvarchar(50) output -- cont el resultat de l'execuci
as
begin
	/*
	Descripci:
	El procediment s'encarrega de modificar un producte existent del sistema, previament vlida 
	que s'hagi emplenat totes les dades i que aquestes siguin correctes
	*/
	
	set nocount on
	
	declare @rsp_trace nvarchar(50) -- cont la resposta del procediment trace
	
	-- obtenim la cadena amb els parametres d'entrada
	declare @params nvarchar(max) = 'identificador_ean13: ' + cast(isnull(@identificador_ean13, '') as nvarchar) +
									'descripcio: ' + @descripcio +
									'preu_brut_actual: ' + cast(@preu_brut_actual as nvarchar) +
									'benefici_net_actual: ' + cast(@benefici_net_actual as nvarchar)

	begin try
		begin transaction
		
		-- validacions
		if(not exists(select * from tw_producte where identificador_ean13 = @identificador_ean13))
			raiserror('Producte no existeix', 16, 1)			
		
		if(not exists(select * from tw_producte where identificador_ean13 = @identificador_ean13 and baixa is null))
			raiserror('Producte ja esta donada de baixa', 16, 1)
		
		if(@descripcio is null or LEN(@descripcio) = 0)
			raiserror('Es requereix una descripci', 16, 1)
		
		if(@preu_brut_actual is null or @preu_brut_actual <= 0)
			raiserror('Es necessari indicar un preu de venda vlid', 16, 1)
			
		if(@benefici_net_actual is null or @benefici_net_actual <= 0)			
			raiserror('El producte a de tindr un benefici', 16, 1)
			
		-- edici
		update tw_producte 
		set descripcio = @descripcio,
			preu_brut_actual = @preu_brut_actual,
			benefici_net_actual = @benefici_net_actual
		where identificador_ean13 = @identificador_ean13
		
		if @@TRANCOUNT > 0
			commit transaction
			
		set @rsp = 'OK'
		
		exec trace 'pm_botigues', @params, 1, @rsp, @rsp_trace output
	end try
	begin catch
	
		if @@TRANCOUNT > 0
			rollback transaction		
		
		declare @error_message nvarchar(max) = error_message()
		set @rsp = 'ERROR ' + error_message()
		
		exec trace 'pm_botigues', @params, 0, @error_message, @rsp_trace output
		
	end catch
end
go

create procedure [dbo].[pb_productes]
	@identificador_ean13 nvarchar(13), -- identificador del producte a donar de baixa
	@rsp nvarchar(50) output -- cont el resultat de l'execuci
as
begin
	/*
	Descripci:
	El procediment s'encarrega de donar de baixa un producte existent del sistema, previament vlida 
	que el producte existeixi i estigui d'alta
	*/
	
	set nocount on
	
	declare @rsp_trace nvarchar(50) -- cont la resposta del procediment trace
	
	-- obtenim la cadena amb els parametres d'entrada
	declare @params nvarchar(max) = 'identificador_ean13: ' + cast(isnull(@identificador_ean13, '') as nvarchar)
	
	begin try
		begin transaction
		
		-- validacions
		if(not exists(select * from tw_producte where identificador_ean13 = @identificador_ean13))
			raiserror('Producte no existeix', 16, 1)
			
		if(not exists(select * from tw_producte where identificador_ean13 = @identificador_ean13 and baixa is null))
			raiserror('Producte ja esta donada de baixa', 16, 1)
			
		-- baixa
		update tw_producte 
		set baixa = getdate()
		where identificador_ean13 = @identificador_ean13
		
		if @@TRANCOUNT > 0
			commit transaction
			
		set @rsp = 'OK'
		
		exec trace 'pb_productes', @params, 1, @rsp, @rsp_trace output
	end try
	begin catch
	
		if @@TRANCOUNT > 0
			rollback transaction
		
		declare @error_message nvarchar(max) = error_message()
		set @rsp = 'ERROR ' + error_message()
		
		exec trace 'pb_productes', @params, 0, @error_message, @rsp_trace output
		
	end catch	
end
go
	
create procedure [dbo].[pa_vendes]
	@id_botiga int, -- identificador de la botiga on es crea la venda
	@identificador_ean13 nvarchar(13), -- identificador del producte que s'utilitza en la venda
	@data_venda date, -- data de la venda
	@hora_venda int, -- hora de la venda
	@quantitat_total int, --quantitat venuda
	@rsp nvarchar(50) output -- cont el resultat de l'execuci
as
begin
	/*
	Descripci:
	El procediment s'encarrega de insertar una nova venda al sistema, previament vlida 
	que s'hagi emplenat totes les dades i que aquestes siguin correctes
	*/
	
	set nocount on
	
	declare @rsp_trace nvarchar(50) -- cont la resposta del procediment trace
	
	-- obtenim la cadena amb els parametres d'entrada
	declare @params nvarchar(max) = 'id_botiga: ' + cast(@id_botiga as nvarchar) +
									'identificador_ean13: ' + isnull(@identificador_ean13, '') + 
									'data_venda: ' + cast(@data_venda as nvarchar) + 
									'hora_venda: ' + cast(@hora_venda as nvarchar) +
									'quantitat_total: ' + cast(@quantitat_total as nvarchar)
	
	begin try
		begin transaction
		
		-- validacions
		if(not exists(select * from tw_botiga where id_botiga = @id_botiga))
			raiserror('Botiga no existeix', 16, 1)
		
		if(not exists(select * from tw_botiga where id_botiga = @id_botiga and baixa is null))
			raiserror('Botiga donada de baixa', 16, 1)
			
		if(not exists(select * from tw_producte where identificador_ean13 = @identificador_ean13))
			raiserror('Producte no existeix', 16, 1)
		
		if(not exists(select * from tw_producte where identificador_ean13 = @identificador_ean13 and baixa is null))
			raiserror('Producte donat de baixa', 16, 1)
			
		if(@data_venda is null)
			raiserror('Se ha de informar la data de la venda', 16, 1)
			
		if(@hora_venda is null or @hora_venda <= 0 or @hora_venda > 24)
			raiserror('L''hora de la venda no es vlida', 16, 1)
			
		if(@quantitat_total is null or @quantitat_total <= 0)
			raiserror('La quantitat no es vlida', 16, 1)
			
		if(exists(select * from tw_vendes where baixa is not null and id_botiga = @id_botiga and identificador_ean13 = @identificador_ean13 and data_venda = @data_venda and hora_venda = @hora_venda))
			raiserror('La venda esta donada de baixa', 16, 1)
			
		if(exists(select * from tw_vendes where id_botiga = @id_botiga and identificador_ean13 = @identificador_ean13 and data_venda = @data_venda and hora_venda = @hora_venda))
			raiserror('Ja existeix una venda amb aquestes caracteristiques', 16, 1)
				
		--clcul de preu total i benefici total
		declare @preu_brut_total decimal(18,2), -- preu del producte acumulat
				@benefici_net_total decimal(18,2) -- benefici total aportat
		
		select @preu_brut_total = (preu_brut_actual * @quantitat_total), 
				@benefici_net_total = (benefici_net_actual * @quantitat_total)
		from tw_producte where identificador_ean13 = @identificador_ean13
		
		-- permetem definir el identity de la taula
		SET IDENTITY_INSERT dbo.tw_vendes ON;
		
		-- insercio
		insert into tw_vendes (id_botiga, identificador_ean13, data_venda, hora_venda, quantitat_total, preu_brut_total, benefici_net_total)
		values(@id_botiga, @identificador_ean13, @data_venda, @hora_venda, @quantitat_total, @preu_brut_total, @benefici_net_total)
		
		if @@TRANCOUNT > 0
			commit transaction
			
		set @rsp = 'OK'
		
		exec trace 'pa_vendes', @params, 1, @rsp, @rsp_trace output
	end try
	begin catch
	
		if @@TRANCOUNT > 0
			rollback transaction
		
		declare @error_message nvarchar(max) = error_message()
		set @rsp = 'ERROR ' + error_message()
		
		exec trace 'pa_vendes', @params, 0, @error_message, @rsp_trace output
		
	end catch
end
go
	
create procedure [dbo].[pm_vendes]
	@id_botiga int, -- identificador de la botiga on es modifica la venda
	@identificador_ean13 nvarchar(13), -- identificador del producte que s'utilitza en la venda
	@data_venda date, -- data de la venda
	@hora_venda int, -- hora de la venda
	@quantitat_total int, --quantitat venuda
	@rsp nvarchar(50) output -- cont el resultat de l'execuci
as
begin
	/*
	Descripci:
	El procediment s'encarrega de modificar una venda existent del sistema, previament vlida 
	que s'hagi emplenat totes les dades i que aquestes siguin correctes
	*/
	
	set nocount on
	
	
	declare @rsp_trace nvarchar(50) -- cont la resposta del procediment trace
	
	-- obtenim la cadena amb els parametres d'entrada
	declare @params nvarchar(max) = 'id_botiga: ' + cast(@id_botiga as nvarchar) +
									'identificador_ean13: ' + isnull(@identificador_ean13, '') + 
									'data_venda: ' + cast(@data_venda as nvarchar) + 
									'hora_venda: ' + cast(@hora_venda as nvarchar) +
									'quantitat_total: ' + cast(@quantitat_total as nvarchar)
	
	begin try
		begin transaction
		
		-- validacions
		if(not exists(select * from tw_botiga where id_botiga = @id_botiga))
			raiserror('Botiga no existeix', 16, 1)
		
		if(not exists(select * from tw_botiga where id_botiga = @id_botiga and baixa is null))
			raiserror('Botiga donada de baixa', 16, 1)
			
		if(not exists(select * from tw_producte where identificador_ean13 = @identificador_ean13))
			raiserror('Producte no existeix', 16, 1)
		
		if(not exists(select * from tw_producte where identificador_ean13 = @identificador_ean13 and baixa is null))
			raiserror('Producte donat de baixa', 16, 1)
			
		if(@data_venda is null)
			raiserror('Se ha de informar la data de la venda', 16, 1)
			
		if(@hora_venda is null or @hora_venda <= 0 or @hora_venda > 24)
			raiserror('L''hora de la venda no es vlida', 16, 1)
		
		if(@quantitat_total is null or @quantitat_total <= 0)
			raiserror('La quantitat no es vlida', 16, 1)
			
		if(exists(select * from tw_vendes where baixa is not null and id_botiga = @id_botiga and identificador_ean13 = @identificador_ean13 and data_venda = @data_venda and hora_venda = @hora_venda))
			raiserror('La venda esta donada de baixa', 16, 1)
			
		if(not exists(select * from tw_vendes where id_botiga = @id_botiga and identificador_ean13 = @identificador_ean13 and data_venda = @data_venda and hora_venda = @hora_venda))
			raiserror('No existeix una venda amb aquestes caracteristiques', 16, 1)
			
		--clcul de preu total i benefici total
		declare @preu_brut_total decimal(18,2), -- preu del producte acumulat
				@benefici_net_total decimal(18,2) -- benefici total aportat
		
		select @preu_brut_total = (preu_brut_actual * @quantitat_total), 
				@benefici_net_total = (benefici_net_actual * @quantitat_total)
		from tw_producte where identificador_ean13 = @identificador_ean13
			
		-- modificaci
		update tw_vendes 
		set quantitat_total = @quantitat_total,
			preu_brut_total = @preu_brut_total,
			benefici_net_total = @benefici_net_total
		where id_botiga = @id_botiga 
			and identificador_ean13 = @identificador_ean13 
			and data_venda = @data_venda 
			and hora_venda = @hora_venda
		
		if @@TRANCOUNT > 0
			commit transaction
			
		set @rsp = 'OK'
		
		exec trace 'pm_vendes', @params, 1, @rsp, @rsp_trace output
	end try
	begin catch
	
		if @@TRANCOUNT > 0
			rollback transaction
		
		declare @error_message nvarchar(max) = error_message()
		set @rsp = 'ERROR ' + error_message()
		
		exec trace 'pm_vendes', @params, 0, @error_message, @rsp_trace output
		
	end catch
end
go

create procedure [dbo].[pb_vendes]
	@id_botiga int, -- identificador de la botiga on es dona de baixa la venda
	@identificador_ean13 nvarchar(13), -- identificador del producte que es dona de baixa la venda
	@data_venda date, -- data de la venda que es vol donar de baixa
	@hora_venda int, -- hora de la venda que es vol donar de baixa
	@rsp nvarchar(50) output -- cont el resultat de l'execuci
as
begin
	/*
	Descripci:
	El procediment s'encarrega de donar de baixa una venda existent del sistema, previament vlida 
	que la venda existeixi i estigui d'alta
	*/
	
	set nocount on
	
	declare @rsp_trace nvarchar(50) -- cont la resposta del procediment trace
	
	-- obtenim la cadena amb els parametres d'entrada
	declare @params nvarchar(max) = 'id_botiga: ' + cast(@id_botiga as nvarchar) +
									'identificador_ean13: ' + isnull(@identificador_ean13, '') + 
									'data_venda: ' + cast(@data_venda as nvarchar) + 
									'hora_venda: ' + cast(@hora_venda as nvarchar)
	
	begin try
		begin transaction
		
		-- validacions
		if(not exists(select * from tw_botiga where id_botiga = @id_botiga))
			raiserror('Botiga no existeix', 16, 1)
		
		if(not exists(select * from tw_botiga where id_botiga = @id_botiga and baixa is null))
			raiserror('Botiga donada de baixa', 16, 1)
			
		if(not exists(select * from tw_producte where identificador_ean13 = @identificador_ean13))
			raiserror('Producte no existeix', 16, 1)
		
		if(not exists(select * from tw_producte where identificador_ean13 = @identificador_ean13 and baixa is null))
			raiserror('Producte donat de baixa', 16, 1)
			
		if(@data_venda is null)
			raiserror('Se ha de informar la data de la venda', 16, 1)
			
		if(@hora_venda is null or @hora_venda <= 0 or @hora_venda > 24)
			raiserror('L''hora de la venda no es vlida', 16, 1)
			
		if(not exists(select * from tw_vendes where id_botiga = @id_botiga and identificador_ean13 = @identificador_ean13 and data_venda = @data_venda and hora_venda = @hora_venda))
			raiserror('No hi ha una venda vlida amb aquestes caracteristiques', 16, 1)
			
		if(not exists(select * from tw_vendes where baixa is null and id_botiga = @id_botiga and identificador_ean13 = @identificador_ean13 and data_venda = @data_venda and hora_venda = @hora_venda))
			raiserror('La venda ja ha estat donada de baixa', 16, 1)
			
		-- baixa
		update tw_vendes 
		set baixa = getdate()
		where id_botiga = @id_botiga 
			and identificador_ean13 = @identificador_ean13 
			and data_venda = @data_venda 
			and hora_venda = @hora_venda
		
		if @@TRANCOUNT > 0
			commit transaction
			
		set @rsp = 'OK'
		
		exec trace 'pb_vendes', @params, 1, @rsp, @rsp_trace output
	end try
	begin catch
	
		if @@TRANCOUNT > 0
			rollback transaction
		
		declare @error_message nvarchar(max) = error_message()
		set @rsp = 'ERROR ' + error_message()
		
		exec trace 'pb_vendes', @params, 0, @error_message, @rsp_trace output
		
	end catch
end
go