use TFC
go

if(exists(select * from sys.objects where object_id = object_id(N'[dbo].[ps_botigues_llistat]')))
	drop procedure ps_botigues_llistat
if(exists(select * from sys.objects where object_id = object_id(N'[dbo].[ps_productes_cataleg]')))
	drop procedure ps_productes_cataleg
if(exists(select * from sys.objects where object_id = object_id(N'[dbo].[ps_benefici_per_dia]')))
	drop procedure ps_benefici_per_dia
go

create procedure [dbo].[ps_botigues_llistat]
	@any int,
	@mes int,
	@rsp nvarchar(50) output
as
begin
	/*
	Descripci:
	Donat un any i un mes concret el llistat de totes les botigues de la cadena incloent per a cada botiga	
	- el nombre total de productes venuts aquell mes
	- el nombre de productes diferentes venuts aquell mes
	- el benefici net total obtingut aquell mes
	- el percentatge de benefici que aporta la botiga en relaci al total de beneficis obtinguts per tota la cadena aquell mes
	- el benefici net dividit pel nombre d'empleats de la botiga
	*/
	set nocount on
	
	begin try
		
		declare @rsp_trace nvarchar(50) -- cont la resposta del procediment trace
		
		-- obtenim la cadena amb els parametres d'entrada
		declare @params nvarchar(max) = 'mes: ' + cast(@mes as nvarchar) +
										', any: ' + cast(@any as nvarchar)
		
		--calculem el total de benefici de la cadena pel periode definit per parmetres
		declare @benefici_total_cadena decimal(18,2)
		select @benefici_total_cadena = SUM(benefici_net_total)
		from tw_vendes
		where MONTH(data_venda) = @mes
			and YEAR(data_venda) = @any
		
		-- obtenim les dades sollicitades
		select tt.id_botiga, tt.productes_venuts, tt.num_productes, tt.benefici_total_mes, ((tt.benefici_total_mes * 100) / @benefici_total_cadena) percentatge_sobre_cadena, (tt.benefici_total_mes / b.nombre_treballadors) benefici_treballador
		from (
			select id_botiga, SUM(quantitat_total) productes_venuts, count(distinct identificador_ean13) num_productes, round(SUM(benefici_net_total), 2) benefici_total_mes
			from tw_vendes
			where MONTH(data_venda) = @mes
				and YEAR(data_venda) = @any
			group by id_botiga
		) tt
			inner join tw_botiga b on b.id_botiga = tt.id_botiga
		group by tt.id_botiga, tt.productes_venuts, tt.num_productes, tt.benefici_total_mes, b.nombre_treballadors
		order by tt.benefici_total_mes

		set @rsp = 'OK'
		
		exec trace 'ps_botigues_llistat', @params, 1, @rsp, @rsp_trace output
	end try
	begin catch
	
		declare @error_message nvarchar(max) = error_message()
		set @rsp = 'ERROR ' + error_message()
		
		exec trace 'ps_botigues_llistat', @params, 0, @error_message, @rsp_trace output
	
	end catch	
end
go

create procedure [dbo].[ps_productes_cataleg]
	@any int,
	@mes int,
	@rsp nvarchar(50) output
as
begin
	/*
	Descripci:
	Donats un any i un mes concret com a parmetres dentrada, el llistat de tots els productes del catleg incloent:	
	- identificador_ean_13
	- nom del producte
	- nombre d'unitats venudes
	- benefici net que ha generat el producte
	- botiga que n'ha venut ms unitats aix com el nombre d'unitats que ha venut
	*/
	set nocount on
	
	begin try
		
		declare @rsp_trace nvarchar(50) -- cont la resposta del procediment trace
		
		-- obtenim la cadena amb els parametres d'entrada
		declare @params nvarchar(max) = 'mes: ' + cast(@mes as nvarchar) +
										', any: ' + cast(@any as nvarchar)
											
		-- comprobem que no estigui fent-se servir la taula temportal
		if (object_id('tempdb..#catalog') is not null)
			raiserror('Ja es troba creada la taula temporal', 16, 1)
			
		-- identifiquem  el periode vlid de venda dels productes
		declare @baixa datetime = cast('01/' + cast(@mes as nvarchar) + '/' + cast(@any as nvarchar) as datetime)
		declare @alta datetime = DATEADD(day, -1, dateadd(month, 1, @baixa))
		
		--creem una taula temporal per tindr la relaci de productes i el valor de venda per botiga
		create table #catalog (identificador_ean13 nvarchar(13), descripcio nvarchar(200), venut_botiga int, benefici_botiga decimal(18,2), id_botiga int)
		
		-- filtrem les dades i les inserim en aquesta taula temporal
		insert into #catalog	
		select p.identificador_ean13, p.descripcio, sum(v.quantitat_total) venut_botiga, sum(v.benefici_net_total) benefici_botiga, v.id_botiga
		from tw_producte p
			left join tw_vendes v on v.identificador_ean13 = p.identificador_ean13
		where p.alta < @alta
			and (p.baixa > @baixa or p.baixa is null)
			and ((month(v.data_venda) = @mes and YEAR(v.data_venda) = @any) 
				or v.data_venda is null)
		group by p.identificador_ean13, p.descripcio, v.id_botiga
		
		-- treiem les dades
		select tt.identificador_ean13, tt.unitats_venudes, tt.benefici_general, id_botiga, tt2.venut_botiga
		from (
			-- obtinim els valors globals del ms per producte
			select identificador_ean13, SUM(venut_botiga) unitats_venudes, SUM(benefici_botiga) benefici_general
			from #catalog
			group by identificador_ean13
		) tt
			inner join (
				-- busquem la tenda amb ms venda per cadascun dels productes
				select identificador_ean13, max(venut_botiga) venut_botiga
				from #catalog c
				group by identificador_ean13
			) tt2 on tt.identificador_ean13 = tt2.identificador_ean13
			inner join #catalog cat on cat.identificador_ean13 = tt2.identificador_ean13 and isnull(cat.venut_botiga, 0) = isnull(tt2.venut_botiga, 0)
		order by tt.benefici_general desc
		
		-- eliminem la taula temporal
		drop table #catalog
		
		set @rsp = 'OK'
		
		exec trace 'ps_productes_cataleg', @params, 1, @rsp, @rsp_trace output
	end try
	begin catch
	
		declare @error_message nvarchar(max) = error_message()
		set @rsp = 'ERROR ' + error_message()		
		
		if (object_id('tempdb..#catalog') is not null and @error_message <> 'Ja es troba creada la taula temporal')
			drop table #catalog
		
		exec trace 'ps_productes_cataleg', @params, 0, @error_message, @rsp_trace output
		
	end catch
end
go

create procedure [dbo].[ps_benefici_per_dia]
	@any int,
	@mes int,
	@rsp nvarchar(50) output
as
begin
	/*
	Descripci:
	Donat un any i un mes concret cal retornar un llistat de tots els dies del mes, incloent-hi per a cada dia	
	- benefici total net obtingut aquell dia per tota la cadena
	- identifiador_ean_13 del producte ms venut, juntament amb les unitats venudes
	- identificador de la botiga que ms benefici net ha generat juntamente amb el valor en euros d'aquest benefici
	*/
	
	set nocount on
	
	begin try
		
		declare @rsp_trace nvarchar(50) -- cont la resposta del procediment trace
		
		-- obtenim la cadena amb els parametres d'entrada
		declare @params nvarchar(max) = 'mes: ' + cast(@mes as nvarchar) +
										', any: ' + cast(@any as nvarchar)
										
		-- comprobem que no estigui fent-se servir la taula temportal
		if (object_id('tempdb..#beneficis_dia') is not null)
			raiserror('Ja es troba creada la taula temporal', 16, 1)
										
		-- creem una taula temporal per emmmagatzemar les dades del periode
		create table #beneficis_dia (dia int, identificador_ean13 nvarchar(13), quantitat int, id_botiga int, benefici decimal(18,2))
		
		-- inserim les dades trobades a la taula temporal
		insert into #beneficis_dia
		select day(data_venda) dia, identificador_ean13, sum(quantitat_total), id_botiga, sum(benefici_net_total)
		from tw_vendes
		where MONTH(data_venda) = @mes
			and YEAR(data_venda) = @any
		group by data_venda, identificador_ean13, id_botiga
					
		select b.dia, benifici_dia, prod.identificador_ean13, prod.quantitat_producte, bot.id_botiga, bot.benefici_botiga
		from #beneficis_dia b
			inner join (
				-- calculem el benefici obtingut per cada dia
				select dia, SUM(benefici) benifici_dia
				from #beneficis_dia
				group by dia
			) ben on ben.dia = b.dia
			inner join (
				-- obtenim una relaci dels productes venuts per cada dia
				select dia, identificador_ean13, sum(quantitat) quantitat_producte 
				from #beneficis_dia
				group by dia, identificador_ean13
			) prod on prod.dia = b.dia and prod.identificador_ean13 = b.identificador_ean13
			inner join (
				-- obtenim la mxima quantitat de productes venuts en un dia (pertany a un producte no de tots)
				select dia, max(quantitat_producte) max_quantitat
				from (
					select dia, identificador_ean13, sum(quantitat) quantitat_producte 
					from #beneficis_dia
					group by dia, identificador_ean13
				) tt
				group by dia
			) tt1 on tt1.dia = prod.dia and tt1.max_quantitat = prod.quantitat_producte		
			inner join (
				-- obtenim una relaci dels beneficis per botiga per cada dia
				select dia, id_botiga, sum(benefici) benefici_botiga 
				from #beneficis_dia
				group by dia, id_botiga
			) bot on bot.dia = b.dia and bot.id_botiga = b.id_botiga
			inner join (
				-- obtenim el mxim benefici en un dia (pertany a una botiga no de totes)
				select dia, max(benefici_botiga) max_benefici
				from (
					select dia, id_botiga, sum(benefici) benefici_botiga 
					from #beneficis_dia
					group by dia, id_botiga
				) tt
				group by dia
			) tt2 on tt2.dia = bot.dia and tt2.max_benefici = bot.benefici_botiga
		group by b.dia, ben.benifici_dia, prod.identificador_ean13, prod.quantitat_producte, bot.benefici_botiga, bot.id_botiga
		
		-- esborrem la taula temporal
		drop table #beneficis_dia
		
		set @rsp = 'OK'
		
		exec trace 'ps_benefici_per_dia', @params, 1, @rsp, @rsp_trace output
	end try
	begin catch
	
		declare @error_message nvarchar(max) = error_message()
		set @rsp = 'ERROR ' + error_message()
				
		if (object_id('tempdb..#beneficis_dia') is not null and @error_message <> 'Ja es troba creada la taula temporal')
			drop table #beneficis_dia
		
		exec trace 'ps_benefici_per_dia', @params, 0, @error_message, @rsp_trace output
		
	end catch
end
go